@@ -191,7 +191,7 @@ class InteractiveShell(cmd.Cmd):
             "  / __ \\/ | / / |  / /  _/ ____/",
             " / / / /  |/ /| | / // // /_    ",
             "/ /_/ / /|  / | |/ // // __/    ",
-            "\\____/_/ |_/  |___/___/_/  v0.1.5",
+            "\\____/_/ |_/  |___/___/_/  v0.1.6",
             "                                ",
         ]
 
@@ -240,25 +240,27 @@ class InteractiveShell(cmd.Cmd):
         while not self._stop_health_check.is_set():
             sock = None
             try:
-                sock = socket.create_connection(
-                    (self.args.host, self.args.port), timeout=5.0
-                )
-
-                # If HTTPS, perform a short TLS handshake check
+                # For HTTPS, use ssl.create_connection for proper TLS handling
                 if getattr(self.args, "https", False):
                     context = ssl.create_default_context()
                     context.minimum_version = ssl.TLSVersion.TLSv1_2
                     context.check_hostname = False
                     context.verify_mode = ssl.CERT_NONE
 
-                    with context.wrap_socket(
-                        sock, server_hostname=self.args.host
-                    ) as ssock:
-                        ssock.settimeout(5)
-                        # Try a minimal handshake
-                        ssock.do_handshake()
+                    # Use ssl.create_connection instead of wrapping existing socket
+                    sock = context.wrap_socket(
+                        socket.socket(socket.AF_INET, socket.SOCK_STREAM),
+                        server_hostname=self.args.host,
+                    )
+                    sock.settimeout(5.0)
+                    sock.connect((self.args.host, self.args.port))
+                else:
+                    # For HTTP, simple TCP connection check
+                    sock = socket.create_connection(
+                        (self.args.host, self.args.port), timeout=5.0
+                    )
 
-                # If we reach here, the connection (and TLS if applicable) succeeded
+                # Connection successful
                 pass
 
             except (
@@ -368,7 +370,7 @@ class InteractiveShell(cmd.Cmd):
 
         return parts
 
-    def _display_grid(self, items, force_recalc=False):
+    def _display_grid(self, items):
         """Display items in grid format matching TAB completion (vertical layout)"""
         if not items:
             return
@@ -393,7 +395,7 @@ class InteractiveShell(cmd.Cmd):
 
         # Column width calculation: use same algorithm as cmd.Cmd.columnize()
         # Python's columnize uses: maxlen + 2 if maxlen > 0 else 2
-        col_width = longest_length + 1 if longest_length > 0 else 2
+        col_width = longest_length + 2 if longest_length > 0 else 2
 
         # Calculate number of columns that fit
         num_cols = max(1, term_width // col_width)
@@ -661,28 +663,17 @@ class InteractiveShell(cmd.Cmd):
                     print("^C")
                     line = ""
 
-            # Handle TAB completion on empty line
-            if line == "\t":
-                print()  # Newline before grid
-                if self.current_service:
-                    self._display_grid(get_service_methods(self.current_service))
-                else:
-                    services = get_device_available_services(self.client)
-                    commands = self.base_commands
-                    self._display_grid(services + commands)
-                line = ""  # Reset line to show prompt again
-
             line = self.precmd(line)
             stop = self.onecmd(line)
             stop = self.postcmd(stop, line)
 
-    def columnize(self, list, displaywidth=80):
+    def columnize(self, items, displaywidth=80):
         """Override columnize to use grid format for TAB completion"""
-        if not list:
+        if not items:
             return
 
         # Use our grid display for TAB completion with coloring enabled
-        self._display_grid(list, force_recalc=True)
+        self._display_grid(items)
 
     def print_topics(self, header, cmds, cmdlen, maxcol):
         """Override print_topics to use grid format for TAB completion"""
@@ -694,7 +685,7 @@ class InteractiveShell(cmd.Cmd):
             self.stdout.write("%s\n" % str(header))
 
         # Use our grid display for TAB completion with coloring enabled
-        self._display_grid(cmds, force_recalc=True)
+        self._display_grid(cmds)
 
     def do_capabilities(self, line):
         """Show device capabilities in service format"""
@@ -933,7 +924,7 @@ class InteractiveShell(cmd.Cmd):
             return
 
         if not operation_name:
-            print(f"Usage: {colorize('type <operation_name>', 'yellow')}")
+            print("Usage: type <operation_name>")
             return
 
         if not hasattr(self.current_service, operation_name):
@@ -1394,7 +1385,7 @@ class InteractiveShell(cmd.Cmd):
             super().do_help(line)
         else:
             help_text = f"""
-{colorize('ONVIF Interactive Shell Commands - v0.1.5', 'cyan')}\n{colorize('https://github.com/nirsimetri/onvif-python', 'white')}
+{colorize('ONVIF Interactive Shell Commands - v0.1.6', 'cyan')}\n{colorize('https://github.com/nirsimetri/onvif-python', 'white')}
 
 {colorize('Basic Commands:', 'yellow')}
   capabilities, caps       - Show device capabilities
